using System;
using Unit4.CollectionsLib;
using Unit4.UtilsLib;
using TurtleLib;
using System.Text;

namespace PolygonLib
{
    public class Polygon
    {
        private Turtle turtle;
        private int nVerts;
        private List<Point> verts;
        
        public Polygon()
        {
            this.nVerts = 0;
            this.verts = new List<Point>();
            this.turtle = new Turtle();
            this.turtle.SetVisible(false);
            this.turtle.SetDelay(0);
            this.turtle.TailDown();
        }
        public Polygon(int nVerts, Point center, double rad)
        {
            this.verts = new List<Point>();
            Node<Point> pos = this.verts.GetFirst();
            double x, y, angle;
            this.nVerts = nVerts;
            angle = 2 * Math.PI / nVerts;
            for (int i = 0; i < nVerts; i++)
            {
                x = center.GetX() + rad * Math.Cos(angle * i);
                y = center.GetY() + rad * Math.Sin(angle * i);
                Point p = new Point(x, y);
                pos = this.verts.Insert(pos, p);
            }
            this.turtle = new Turtle();
            this.turtle.SetVisible(false);
            this.turtle.SetDelay(0);
            this.turtle.TailDown();
            this.Draw();
        }
        public void AddPoint(Point p)
        {
            Node<Point> prev = null;
            Node<Point> pos = this.verts.GetFirst();
            while (pos != null)
            {
                prev = pos;
                pos = pos.GetNext();
            }
            this.verts.Insert(prev, p);
            this.nVerts++;
            this.turtle.Clear();
            this.Draw();
        }
        public List<Point> GetAllPoints()
        {
            return this.verts;
        }
        public bool Exists(Point p)
        {
            Node<Point> pos = this.verts.GetFirst();
            Point curP;
            while (pos != null)
            {
                curP = pos.GetInfo();
                if (curP.IsEquals(p))
                    return true;
                pos = pos.GetNext();
            }
            return false;
        }
        public void Move(double dx, double dy)
        {
            Node<Point> pos = this.verts.GetFirst();
            this.turtle.Clear();
            Point p;
            while (pos != null)
            {
                p = pos.GetInfo();
                p.SetX(p.GetX() + dx);
                p.SetY(p.GetY() + dy);
                pos = pos.GetNext();
            }
            this.Draw();
        }
        public override string ToString()
        {
            return "Vertices of Polygon:" + this.verts.ToString();
        }
        private void Draw()
        {
            if (this.nVerts == 0)
                return;
            double x, y;
            Node<Point> pos = this.verts.GetFirst();
            // Move to first point 
            this.turtle.TailUp();
            x = pos.GetInfo().GetX();
            y = pos.GetInfo().GetY();
            this.turtle.MoveTo(x, y);
            this.turtle.TailDown();
            //Move the turtle forward steps times along its current heading.
            pos = pos.GetNext();
            while (pos != null)
            {
                x = pos.GetInfo().GetX();
                y = pos.GetInfo().GetY();
                this.turtle.MoveTo(x, y);
                pos = pos.GetNext();
            }
            // Move to First Point
            pos = this.verts.GetFirst();
            x = pos.GetInfo().GetX();
            y = pos.GetInfo().GetY();
            this.turtle.MoveTo(x, y);
            

        }
        //////////////////////////////////////////////////////////////////////////////////////
        public Point Center()
        {
            double x = 0;
            double y = 0;
            Node<Point> pos = this.verts.GetFirst();
            for (int i = 0; i < this.nVerts; i++)
            {
                x += pos.GetInfo().GetX();
                y += pos.GetInfo().GetY();
                pos = pos.GetNext();
            }
            return new Point(x / this.nVerts, y / this.nVerts);
        }
        public void Scale(Point c, double sX, double sY)
        {
            double x;
            double y;
            Node<Point> pos = this.verts.GetFirst();
            for (int i = 0; i < this.nVerts; i++)
            {
                x = (pos.GetInfo().GetX() - c.GetX()) * sY + c.GetX();
                y = (pos.GetInfo().GetY() - c.GetY()) * sY + c.GetY();
                pos.GetInfo().SetX(x);
                pos.GetInfo().SetY(y);
                pos = pos.GetNext();
            }
            this.turtle.Clear();
            this.Draw();

        }
        public void Scale(double sX, double sY)
        {
            this.Scale(this.Center(), sX, sY);
            
            
        }

        public void Rotate(Point c, double angle)
        {
            angle = (angle * Math.PI ) / 180;
            double cs = Math.Cos(angle);
            double sn = Math.Sin(angle);
            double x;
            double y;
            Node<Point> pos = this.verts.GetFirst();
            for (int i = 0; i < this.nVerts; i++)
            {
                x = (pos.GetInfo().GetX() - c.GetX());
                y = (pos.GetInfo().GetY() - c.GetY());
                pos.GetInfo().SetX(x * cs - y * sn + c.GetX());
                pos.GetInfo().SetY(x * sn + y * cs + c.GetY());
                pos = pos.GetNext();
            }
            this.turtle.Clear();
            this.Draw();
 
        }
        public void Rotate(double angle)
        {
            this.Rotate(this.Center(), angle);
        }
    }
}
