using System;
using Games.TurtleLib;


namespace TurtleLib
{
    public class Turtle : Games.TurtleLib.Turtle
    {
        // (x, y) position of turtle
        private double x = 0;
        private double y = 0;
        // heading of turtle in radians
        private Double theta = 0;
        public Turtle() : base()
        {
            this.x = 0;
            this.y = 0;
            this.theta = 0;
        }
        public new void TurnLeft(double degrees)
        {
            double radians;
            double twoPI;

            twoPI = 2 * Math.PI;
            radians = twoPI * degrees / 360;
            this.theta -= radians;
            while (this.theta < 0)
                this.theta += twoPI;
            while (this.theta >= twoPI)
                this.theta -= twoPI;
            base.TurnLeft(degrees);
        }
        public new void TurnRight(double degrees)
        {
            this.TurnLeft(-degrees);
        }
        public new void MoveForward(double steps)
        {
            double sn, cs;
            double dx, dy;
            sn = Math.Sin(this.theta);
            cs = Math.Cos(this.theta);
            dx = steps * sn;
            dy = -steps * cs;
            this.x += steps * sn;
            this.y -= steps * cs;
            base.MoveForward(steps);
        }
        public new void MoveBackward(double steps)
        {
            this.MoveForward(-steps);
        }
        public double GetX()
        {
            return this.x;
        }
        public double GetY()
        {
            return -this.y;
        }
        /// <summary>
        /// Gets this turtle's heading.
        /// </summary>
        /// <returns>turtle's heading</returns>
        public double GetHeading()
        {
            double heading = 180 * this.theta / Math.PI;
            if (heading > 180)
                heading = heading - 360;
            else
                if(heading < -180)
                    heading = heading + 360;
            return heading;
        }
        /// Sets this turtle's heading for a specified degree. The current heading is irrelevant.
        /// If the specified degree is negative, then it is a clockwise direction
        /// </summary>
        /// <param name="degree"> the new orientation of this turtle </param>
        public void SetHeading(double degree)
        {
            this.TurnRight(degree - this.GetHeading());
        }
        private static double GetHeading(double dx, double dy)
        {
            double alfa;
            if (dy == 0)
            {
                if (dx > 0)
                    return 90;
                else
                    if (dx < 0)
                        return -90;
                return 0;
            }
            alfa = Math.Atan(dx / dy);
            if (dy < 0)
                alfa = alfa + Math.PI;
            // Convert From Radians To degrees
            return alfa * 180 / Math.PI;

        }
        /// <summary>
        /// Moves this turtle to the specified target (x, y) position. The turtle's orientation will be adjusted to 
        /// face the the target (x,y) position, regardless where this turtle is facing before this method is called.
        /// </summary>
        /// <param name="targetX">the x coordinate of the target point</param>
        /// <param name="targetY">the y coordinate of the target point</param>
        public void MoveTo(double targetX, double targetY)
        {
            double dx = targetX - this.x;
            double dy = targetY + this.y;
            double heading = Turtle.GetHeading(dx, dy);
            this.SetHeading(heading);
            this.MoveForward(Math.Sqrt(dx*dx + dy*dy));
        }
        public void MoveHome()
        {
            double heading = this.GetHeading();
            //double heading = 180 * this.theta / Math.PI;
            this.TailUp();
            this.MoveTo(0, 0);
            this.SetHeading(heading);
            
            
        }
    }
}
